Object subclass: #AbstractMatrix
	instanceVariableNames: 'elementType height width '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Numeric Linear Algebra'!
!AbstractMatrix commentStamp: '<historical>' prior: 0!
I am the root of a hierarchy of Matrices, suitable for Numeric Lineal Algebra.!


!AbstractMatrix methodsFor: 'accessing' stamp: 'jmv 3/19/2002 20:33'!
at: j putColumn: aVector
	"aVector must be a column vector, i.e. a matrix with width 1"

	1 to: self height do: [ :i |
		self atRow: i column: j put: (aVector atRow: i column: 1) ]! !

!AbstractMatrix methodsFor: 'accessing' stamp: 'jmv 3/19/2002 20:34'!
at: i putRow: aVector
	"aVector must be a row vector, i.e. a matrix with height 1"

	1 to: self width do: [ :j |
		self atRow: i column: j put: (aVector atRow: 1 column: j) ]! !

!AbstractMatrix methodsFor: 'accessing' stamp: 'JMV 11/23/2001 15:23'!
atRow: i column: j
	"i and j go from 1 to size (height or width)"

	^self subclassResponsibility! !

!AbstractMatrix methodsFor: 'accessing' stamp: 'JMV 11/23/2001 15:22'!
atRow: i column: j put: element
	"i and j go from 1 to size (height or width)"

	^self subclassResponsibility! !

!AbstractMatrix methodsFor: 'accessing' stamp: 'jmv 4/25/2002 19:06'!
columnAt: j

	| result |
	result _ Matrix newRows: self height columns: 1.
	1 to: self height do: [ :i |
		result atRow: i column: 1 put: (self atRow: i column: j) ].
	^result! !

!AbstractMatrix methodsFor: 'accessing' stamp: 'jmv 3/19/2002 20:05'!
diagonal

	| x |
	x _ Matrix newVectorSize: self height.
	1 to: self height do: [ :i |
		x atRow: i column: 1 put: (self atRow: i column: i ) ].
	^x! !

!AbstractMatrix methodsFor: 'accessing' stamp: 'jmv 2/8/2002 16:24'!
height
	^height! !

!AbstractMatrix methodsFor: 'accessing' stamp: 'jmv 3/19/2002 19:47'!
isSquare

	^self height = self width! !

!AbstractMatrix methodsFor: 'accessing' stamp: 'jmv 4/25/2002 19:06'!
rowAt: i

	| result |
	result _ Matrix newRows: 1 columns: self width.
	1 to: self width do: [ :j |
		result atRow: 1 column: j put: (self atRow: i column: j) ].
	^result! !

!AbstractMatrix methodsFor: 'accessing' stamp: 'jmv 3/20/2002 17:24'!
subMatrixTopLeft: aPoint bottomRight: bPoint

	| result |
	result _ Matrix newRows: bPoint y - aPoint y + 1 columns: bPoint x - aPoint x + 1.
	1 to: result height do: [ :i |
		1 to: result width do: [ :j |
			result atRow: i column: j put:
				(self atRow: i + aPoint y - 1 column: j + aPoint x - 1) ] ].
	^result! !

!AbstractMatrix methodsFor: 'accessing' stamp: 'jmv 2/8/2002 16:24'!
width
	^width! !


!AbstractMatrix methodsFor: 'aritmethic' stamp: 'jmv 4/10/2002 16:50'!
* aMatrixOrNumber

	| result |
	(aMatrixOrNumber isKindOf: AbstractMatrix) ifTrue: [
		self width = aMatrixOrNumber height ifFalse: [ ^self error: 'Matrix sizes do not match' ].
		result _ Matrix newRows: self height columns: aMatrixOrNumber width.
		self fill: result withResultOfMultiplyBy: aMatrixOrNumber ]
	ifFalse: [
		result _ self copy.
		result replaceValues: [ :value | value * aMatrixOrNumber ] ].
	^result
	! !

!AbstractMatrix methodsFor: 'aritmethic' stamp: 'jmv 4/10/2002 16:53'!
+ aMatrixOrNumber

	| result |
	(aMatrixOrNumber isKindOf: AbstractMatrix) ifTrue: [
		(self width = aMatrixOrNumber width) & (self height = aMatrixOrNumber height)
			ifFalse: [ ^self error: 'Matrix sizes do not match' ].
		result _ Matrix newRows: self height columns: self width.
		self fill: result withResultOfSumWith: aMatrixOrNumber ]
	ifFalse: [
		result _ self copy.
		result replaceValues: [ :value | value + aMatrixOrNumber ] ].
	^result
	! !

!AbstractMatrix methodsFor: 'aritmethic' stamp: 'jmv 4/10/2002 18:02'!
- aMatrixOrNumber

	| result |
	(aMatrixOrNumber isKindOf: AbstractMatrix) ifTrue: [
		(self width = aMatrixOrNumber width) & (self height = aMatrixOrNumber height)
			ifFalse: [ ^self error: 'Matrix sizes do not match' ].
		result _ Matrix newRows: self height columns: self width.
		self fill: result withResultOfDifferenceWith: aMatrixOrNumber ]
	ifFalse: [
		result _ self copy.
		result replaceValues: [ :value | value - aMatrixOrNumber ] ].
	^result
	! !

!AbstractMatrix methodsFor: 'aritmethic' stamp: 'jmv 4/10/2002 18:01'!
fill: resultMatrix withResultOfDifferenceWith: aMatrix

	| elem |
	1 to: resultMatrix height do: [ :i |
		1 to: resultMatrix width do: [ :j |
			elem _ ((self atRow: i column: j) - (aMatrix atRow: i column: j)).
			resultMatrix atRow: i column: j put: elem ] ]! !

!AbstractMatrix methodsFor: 'aritmethic' stamp: 'jmv 4/10/2002 16:49'!
fill: resultMatrix withResultOfMultiplyBy: aMatrix
	"Fill resultMatrix with the result of self * aMatrix"

	| elem |
	1 to: resultMatrix height do: [ :i |
		1 to: resultMatrix width do: [ :j |
			elem _ 0.
			1 to: self width do: [ :k |
				elem _ elem + ((self atRow: i column: k) * (aMatrix atRow: k column: j)) ].
			resultMatrix atRow: i column: j put: elem ] ]! !

!AbstractMatrix methodsFor: 'aritmethic' stamp: 'jmv 4/10/2002 16:53'!
fill: resultMatrix withResultOfSumWith: aMatrix

	| elem |
	1 to: resultMatrix height do: [ :i |
		1 to: resultMatrix width do: [ :j |
			elem _ ((self atRow: i column: j) + (aMatrix atRow: i column: j)).
			resultMatrix atRow: i column: j put: elem ] ]! !


!AbstractMatrix methodsFor: 'eigenvalues and eigenvectors' stamp: 'jmv 4/10/2002 16:51'!
eigenvaluesByQR
	"This implementation only works if all the eigenvalues are real."
	"(Matrix fromArrayOfArrays: #(
		#(6 4 4 1) 
		#(4 6 1 4) 
		#(4 1 6 4)
		#(1 4 4 6)
	)) eigenvaluesByQR"

	| h q r loopCount |
	h _ self hessembergSuperior.
	q _ Matrix newRows: self height columns: self width.
	r _ Matrix newRows: self height columns: self width.
	loopCount _ 0.
	[ h isFirstSubdiagonalZero ] whileFalse: [
		h storeQRDecompositionOfHessemberSuperiorOnQ: q r: r.
		r fill: h withResultOfMultiplyBy: q.
		loopCount _ loopCount + 1.
		loopCount > 1000 ifTrue: [^self error: 'Could not find real eigenvalues'] ].
	^h diagonal! !

!AbstractMatrix methodsFor: 'eigenvalues and eigenvectors' stamp: 'jmv 3/19/2002 19:55'!
hessembergSuperior
	"Answers a new matrix that is the Hessember Superior transform of the receiver.
	A Hessember Superior matrix has zero entries below the first subdiagonal.
	The new matrix has the same eigenvalues as the receiver."

	| result u |
	self isSquare ifFalse: [ ^self error: 'Only for Square Matrices' ].
	result _ self copy.
	u _ Matrix newRows: self height columns: self width.
	1 to: self width-2 do: [ :j |
		(result storeHouseholderTransformOn: u column: j forQR: false) ifTrue: [
			result _ u * result * u.
		].
	].
	^ result
! !

!AbstractMatrix methodsFor: 'eigenvalues and eigenvectors' stamp: 'jmv 4/15/2002 09:38'!
maxEigenvalueAndEigenvectorbyPowerMethod
	| eigenvectorAprox prevAprox swapAux eigenvalueAprox |
	
	self isSquare ifFalse: [ ^self error: 'Only for Square Matrices' ].
	eigenvectorAprox _ Matrix newColumnVectorSize: self width.
	prevAprox _ Matrix newColumnVectorSize: self width.

	1 to: self width do: [ :i |
		eigenvectorAprox atRow: i column: 1 put: 1 ].

	[ (eigenvectorAprox - prevAprox) norm2Squared > 0.001 ] whileTrue: [
		swapAux _ prevAprox.
		prevAprox _ eigenvectorAprox.
		eigenvectorAprox _ swapAux.
		self fill: eigenvectorAprox withResultOfMultiplyBy: prevAprox.
		eigenvalueAprox _ eigenvectorAprox norm2Squared sqrt.
		eigenvectorAprox replaceValues: [ :value | value / eigenvalueAprox ].
	].
	^Array with: eigenvalueAprox with: eigenvectorAprox.
! !

!AbstractMatrix methodsFor: 'eigenvalues and eigenvectors' stamp: 'jmv 3/19/2002 19:56'!
preMultiplyByGivensRotationRowI: i rowK: k titaCosine: c titaSine: s
	"Modify the receiver, doing a premultiplication by a Givens rotation of angle tita,
	affecting rows i and k."

	| elemI elemK |
	1 to: self width do: [ :j |
		elemI _ self atRow: i column: j.
		elemK _ self atRow: k column: j.
		self atRow: i column: j put: c*elemI - (s*elemK).
		self atRow: k column: j put: s*elemI + (c*elemK) ].! !

!AbstractMatrix methodsFor: 'eigenvalues and eigenvectors' stamp: 'jmv 3/19/2002 23:34'!
storeHouseholderTransformOn: aMatrix column: j forQR: forQR
	"Modifies entries on aMatrix to make it the Householder transforms
	that puts zeroes at column i of the receiver. If forQR is false, the
	product of aMatrix * self is Hessemberg superior, otherways its a
	triangular matrix. "

	| x xNorm2Squared v vNorm2Squared element i |
	i _ j - (forQR ifTrue: [ 1 ] ifFalse: [ 0 ]).
	x _ Matrix newVectorSize: self height-i.
	xNorm2Squared _ 0.
	1 to: x height do: [ :ii |
		element _ self atRow: ii+i column: j.
		xNorm2Squared _ xNorm2Squared + element squared.
		x atRow: ii column: 1 put: element ].
	v _ x.
	"If column already has zeros, do nothing"
	xNorm2Squared = 0.0 ifTrue: [ ^false ].
	"If column already has zeros, do nothing. If forQR = false, then the first element in x
	could not be zero, and anyway there's nothing to do."
	(forQR not and: [ xNorm2Squared = (x atRow: 1 column: 1) squared ]) ifTrue: [ ^false ].

	v atRow: 1 column: 1 put: (v atRow: 1 column: 1) + xNorm2Squared sqrt.
	vNorm2Squared _ v norm2Squared.

	1 to: i do: [ :ii |
		aMatrix atRow: ii column: ii put: 1.
		ii+1 to: aMatrix width do: [ :jj |
			aMatrix atRow: ii column: jj put: 0.
			aMatrix atRow: jj column: ii put: 0 ] ].
	1 to: x height do: [ :ii |
		1 to: x height do: [ :jj |
			aMatrix atRow: ii+i column: jj+i put: 
				(ii==jj ifTrue:[1] ifFalse:[0]) - 
				(2.0 / vNorm2Squared * (v atRow: ii column: 1) * (v atRow: jj column:1) ) ] ].
	^true! !

!AbstractMatrix methodsFor: 'eigenvalues and eigenvectors' stamp: 'jmv 3/19/2002 19:58'!
storeQRDecompositionOfHessemberSuperiorOnQ: q r: r
	"Works only if the receiver is a Hessemberg superior matrix."

	| elementAtJ elementBelowJ aux c s |
	q fillWithIdentity.
	r fillWith: self.
	1 to: self width-1 do: [ :j |
		elementAtJ _ r atRow: j column: j.
		elementBelowJ _ r atRow: j+1 column: j.
		elementBelowJ = 0 ifFalse: [
			aux _ ((elementAtJ*elementAtJ) + (elementBelowJ*elementBelowJ)) sqrt.
			c _ elementAtJ / aux.
			s _ 0 - elementBelowJ / aux.
			r preMultiplyByGivensRotationRowI: j rowK: j+1 titaCosine: c titaSine: s.
			q preMultiplyByGivensRotationRowI: j rowK: j+1 titaCosine: c titaSine: s ] ].
	q transpose.! !


!AbstractMatrix methodsFor: 'filling' stamp: 'JMV 11/23/2001 15:47'!
fillDiagonalWith: aVectorOrArray

	1 to: width do: [ :ii |
		self atRow: ii column: ii put: (aVectorOrArray at: ii) ].! !

!AbstractMatrix methodsFor: 'filling' stamp: 'JMV 11/23/2001 15:50'!
fillDiagonalWithIdentity

	| one |
	one _ self zero + 1.
	1 to: width do: [ :ii |
		self atRow: ii column: ii put: one ]! !

!AbstractMatrix methodsFor: 'filling' stamp: 'JMV 11/23/2001 15:48'!
fillWith: aMatrix

	1 to: (height min: aMatrix height) do: [ :i |
		1 to: (width min: aMatrix width) do: [ :j |
			self atRow: i column: j put: (aMatrix atRow: i column: j) ] ]! !

!AbstractMatrix methodsFor: 'filling' stamp: 'jmv 7/8/2002 13:57'!
fillWithArrayOfArrays: anArray
	"Matrix fromArrayOfArrays: #(
		#(1 2 0 0 0 0) 
		#(2 4 1 0 0 -4) 
		#(0 1 1 1 0 0)
		#(0 0 1 1 1 0)
		#(0 0 0 1 1 1)
		#(0 1 0 0 1 1)
	)."

	1 to: (height min: anArray size) do: [ :i |
		1 to: (width min: anArray first size) do: [ :j |
			self atRow: i column: j put: ((anArray at: i) at: j) ] ]! !

!AbstractMatrix methodsFor: 'filling' stamp: 'JMV 11/23/2001 15:51'!
fillWithIdentity

	self fillWithZero.
	self fillDiagonalWithIdentity.! !

!AbstractMatrix methodsFor: 'filling' stamp: 'jmv 3/22/2002 13:16'!
fillWithIdentityTopAndBottomRows: aNumber

	| zero one heightPlusOne ii |
	zero _ self zero.
	one _ zero + 1.
	heightPlusOne _ height + 1.
	1 to: aNumber do: [ :i |
		ii _ heightPlusOne - i.
		1 to: width do: [ :j |
			self atRow: i column: j put: zero.
			self atRow: ii column: j put: zero ].
		self atRow: i column: i put: one.
		self atRow: ii column: ii put: one.
	]! !

!AbstractMatrix methodsFor: 'filling' stamp: 'JMV 11/23/2001 15:50'!
fillWithZero

	| zero |
	zero _ self zero.
	1 to: height do: [ :i |
		1 to: width do: [ :j |
			self atRow: i column: j put: zero ] ]! !


!AbstractMatrix methodsFor: 'initialization' stamp: 'JMV 11/23/2001 15:06'!
initType: aSymbol rows: n columns: m

	elementType _ aSymbol.
	height _ n.
	width _ m.
	self initializeElements.! !


!AbstractMatrix methodsFor: 'linear systems operations' stamp: 'jmv 2/8/2002 16:22'!
permuteRow: i withRow: k

	| a b |
	i = k ifFalse: [
		1 to: self width do: [ :j |
			a _ self atRow: k column: j.
			b _ self atRow: i column: j.
			self atRow: k column: j put: b.
			self atRow: i column: j put: a ] ]! !

!AbstractMatrix methodsFor: 'linear systems operations' stamp: 'jmv 2/8/2002 16:22'!
rowWithMaxInColumn: j startingAtRow: jStart

	| rowWithMax |
	rowWithMax _ jStart.
	jStart+1 to: self height do: [ :i |
		(self atRow: i column: j) abs > (self atRow: rowWithMax column: j) abs ifTrue: [
			rowWithMax _ i ] ].
	^rowWithMax! !

!AbstractMatrix methodsFor: 'linear systems operations' stamp: 'jmv 3/19/2002 21:19'!
solveLinearSystem
	| m n x sum epsilon |
	"Solve the sistem A x = b where the receiver has this form | Ab |,
	the last column of the receiver contains the independent term b
	This method modifies the receiver!!."
	"(Matrix fromArrayOfArrays: #(
		#(1 2 0 0 0 0 3) 
		#(2 4 1 0 0 -4 3) 
		#(0 1 1 1 0 0 3)
		#(0 0 1 1 1 0 3)
		#(0 0 0 1 1 1 3)
		#(0 1 0 0 1 1 3)
	)) solveLinearSystem"

	m _ self height.
	n _ self width.
	x _ Matrix newVectorSize: n-1.

	"Check we have enough equations"
	m < (n-1) ifTrue: [
		^self error: 'This system does not have a single solution' ].

	"We can safely do comparisions with this epsilon for assuring that an element is close to zero,
	because we first normalize the rows."
	self normalizeRows.
	epsilon _ 1e-7.

	self triangleWithPartialPivoting.

	"Checks"
	"If the only coeficient of the last equation is zero, we have infinite solutions."
	((self atRow: n-1 column: n-1) abs < epsilon) & ((self atRow: n-1 column: n) abs < epsilon) ifTrue: [
		^self error: 'This system does not have a single solution' ].
	"If the only coeficient of the last equation is zero, but not the independent term,
	the system is incompatible."
	(self atRow: n-1 column: n-1) abs < epsilon ifTrue: [
		^self error: 'This system is incompatible (it does not have solution)' ].
	"We have too much equations, and they did not go away. Incompatible system."
	(m > (n-1) and: [ (self atRow: n column: n-1) abs > epsilon ]) ifTrue: [
		^self error: 'This system does is incompatible (it does not have solution)' ].
	"We have too much equations, and they left their independent terms."
	((n to: m) inject: true into: [ :previousValue :k |
		previousValue & ((self atRow: k column: n) abs < epsilon ) ]) ifFalse: [
			^self error: 'This system does is incompatible (it does not have solution)' ].

	"Do backward substitution"
	n-1 to: 1 by: -1 do: [ :i |
		sum _ self atRow: i column: n.
		n-1 to: i+1 by: -1 do: [ :k |
			sum _ sum - ((self atRow: i column: k) * (x atRow: k column: 1)) ].
		x atRow: i column: 1 put: sum / (self atRow: i column: i) ].
	^x! !

!AbstractMatrix methodsFor: 'linear systems operations' stamp: 'jmv 4/10/2002 20:14'!
substractRow: i multipliedBy: factor to: k startingAtColumn: jStart

	jStart to: self width do: [ :j |
		self atRow: k column: j put:
			(self atRow: k column: j) - ((self atRow: i column: j) * factor) ]! !

!AbstractMatrix methodsFor: 'linear systems operations' stamp: 'jmv 4/10/2002 20:14'!
triangleWithPartialPivoting
	"Triangle self, and answer the row permutation count"
	"(Matrix fromArrayOfArrays: #(
		#(1 2 0 0 0 0 3) 
		#(2 4 1 0 0 -4 3) 
		#(0 1 1 1 0 0 3)
		#(0 0 1 1 1 0 3)
		#(0 0 0 1 1 1 3)
		#(0 1 0 0 1 1 3)
	)) triangleWithPartialPivoting; yourself"

	| j jdelta k factor permutationCount |
	permutationCount _ 0.
	jdelta _ 0.
	1 to: self height-1 do: [ :i |

		[
			j _ i + jdelta.
			j <= self width ifFalse: [ ^self ].
 			"Look for the pivot for column j, from row i down. Call it row k"
			k _ self rowWithMaxInColumn: j startingAtRow: i.
			((self atRow: k column: j) = 0) & (j < self width) ] whileTrue: [ jdelta _ jdelta + 1 ].

		"Permute rows k and i"
		i = k ifFalse: [
			self permuteRow: i withRow: k.
			permutationCount _ permutationCount + 1 ].

      	"Substract the row i to all the ones below it"
		i+1 to: self height do: [ :ii |
			(self atRow: ii column: j) = 0.0 ifFalse: [
				factor _ (self atRow: ii column: j) / (self atRow: i column: j).
				"Only after column j"
				self atRow: ii column: j put: 0.
				self substractRow: i multipliedBy: factor to: ii startingAtColumn: j+1 ] ]
	].
	^ permutationCount! !


!AbstractMatrix methodsFor: 'misc. operations' stamp: 'jmv 4/10/2002 17:20'!
asBandMatrixZeroThreshold: zt
	| result maxDist |

	self isSquare ifFalse: [ ^self error: 'Only for Square Matrices' ].

	maxDist _ 0.
	1 to: self height do: [ :i |
		1 to: self width do: [ :j |
			(self atRow: i column: j) abs > zt ifTrue: [
(i-j) abs > maxDist ifTrue: [
Transcript cr; show: (i@j) printString, ' ', (self atRow: i column: j) printString ].
				maxDist _ maxDist max: (i-j) abs ] ] ].

	result _ BandMatrix newRowsAndColumns: self height bandWidth: maxDist*2+1.
	result fillBandWith: self.
	^result! !

!AbstractMatrix methodsFor: 'misc. operations' stamp: 'jmv 3/19/2002 19:49'!
copy

	^self copyTwoLevel! !

!AbstractMatrix methodsFor: 'misc. operations' stamp: 'jmv 3/19/2002 20:26'!
determinant
	"(Matrix fromArrayOfArrays: #(
		#(2 0 0 3) 
		#(1 1 0 0)
		#(1 1 1 0) 
		#(5 1 1 9)
	)) determinant"

	| aux rowPermutationCount result |
	self isSquare ifFalse: [ ^self error: 'Only for Square Matrices' ].
	aux _ self copy.
	rowPermutationCount _ aux triangleWithPartialPivoting.
	result _ -1 raisedToInteger: rowPermutationCount.
	1 to: self width do: [ :i |
		result _ result * (aux atRow: i column: i) ].
	^result! !

!AbstractMatrix methodsFor: 'misc. operations' stamp: 'jmv 4/10/2002 20:16'!
inverse
	"Answers the inverse matrix"
	"(Matrix fromArrayOfArrays: #(
		#(1 2 0 0 0 0) 
		#(2 4 1 0 0 -4) 
		#(0 1 1 1 0 0)
		#(0 0 1 1 1 0)
		#(0 0 0 1 1 1)
		#(0 1 0 0 1 1)
	)) inverse"

	| n bigMatrix result epsilon |
	self isSquare ifFalse: [ ^self error: 'Only for Square Matrices' ].
	n _ self height.
	result _ Matrix newRows: n columns: n.
	bigMatrix _ Matrix newRows: n columns: n * 2.
	bigMatrix fillWith: self.
	1 to: self width do: [ :i |
		bigMatrix atRow: i column: n + i  put: 1.
	].

	"We can safely do comparisions with this epsilon for assuring that an element is close to zero,
	because we first normalize the rows."
	bigMatrix normalizeRows.
	epsilon _ 1e-7.
	bigMatrix triangleWithPartialPivoting.

	"If the only coeficient of the last equation is zero, but not the independent term,
	the system is incompatible."
	(bigMatrix atRow: n column: n) abs < epsilon ifTrue: [
		^self error: 'This matrix does not have an inverse' ].

	"Go backwards to make the left half be the identity.
	On the right half, we'll have the inverse"
	n to: 1 by: -1 do: [ :i |
		bigMatrix multiplyRightPartOfRow: i by: 1.0 / (bigMatrix atRow: i column: i).
		i-1 to: 1 by: -1 do: [ :ii |
			(bigMatrix atRow: ii column: i) = 0 ifFalse: [
				bigMatrix
					substractRow: i 
					multipliedBy: (bigMatrix atRow: ii column: i)
					to: ii 
					startingAtColumn: ii
			].
		].
	].

	"Take result"
	1 to: n do: [ :i |
		1 to: n do: [ :j |
			result atRow: i column: j put: (bigMatrix atRow: i column: j+n) ] ].

	^result! !

!AbstractMatrix methodsFor: 'misc. operations' stamp: 'jmv 4/10/2002 19:26'!
inverse2
	"Answers the inverse matrix"
	"(Matrix fromArrayOfArrays: #(
		#(1 2 0 0 0 0) 
		#(2 4 1 0 0 -4) 
		#(0 1 1 1 0 0)
		#(0 0 1 1 1 0)
		#(0 0 0 1 1 1)
		#(0 1 0 0 1 1)
	)) inverse"

	| bigMatrix result |
	self isSquare ifFalse: [ ^self error: 'Only for Square Matrices' ].
	result _ Matrix newRows: self height columns: self width.
	bigMatrix _ Matrix newRows: self height columns: self width + 1.
	1 to: self width do: [ :i |
		bigMatrix fillWith: self.
		1 to: self width do: [ :j | bigMatrix atRow: j column: self width+1  put: 0 ].
		bigMatrix atRow: i column: self width + 1  put: 1.
		result at: i putColumn: bigMatrix solveLinearSystem ].
	^result! !

!AbstractMatrix methodsFor: 'misc. operations' stamp: 'jmv 3/20/2002 14:50'!
isFirstSubdiagonalZero
	"Answer true if the values of the first diagonal are small enough 
	(when compared with the diagonal) to be considered zero."
	"(Matrix fromArrayOfArrays: #(
		#(2 0 0 3) 
		#(0 1 0 0)
		#(0 10 100000000 0) 
		#(0 0 0 9)
	)) isFirstSubdiagonalZero"

	| measure floatHolder |
	floatHolder _ FloatArray new: 2.
	^(2 to: self width) inject: true into: [ :previousValue :j |
		previousValue and: [ 
			measure _ (self atRow: j column: j) * 100.
			measure == 0.0 ifTrue: [ measure _ self diagonal max ].
			floatHolder at: 1 put: (self atRow: j column: j-1)+measure.
			floatHolder at: 2 put: measure.
			(floatHolder at: 1) = (floatHolder at: 2) ] ]! !

!AbstractMatrix methodsFor: 'misc. operations' stamp: 'jmv 4/10/2002 19:50'!
multiplyColumn: j by: factor

	1 to: height do: [ :i |
		self atRow: i column: j put: (self atRow: i column: j) * factor ]! !

!AbstractMatrix methodsFor: 'misc. operations' stamp: 'jmv 4/10/2002 19:54'!
multiplyRightPartOfRow: i by: factor

	i to: width do: [ :j |
		self atRow: i column: j put: (self atRow: i column: j) * factor ]! !

!AbstractMatrix methodsFor: 'misc. operations' stamp: 'jmv 4/10/2002 19:49'!
multiplyRow: i by: factor

	1 to: width do: [ :j |
		self atRow: i column: j put: (self atRow: i column: j) * factor ]! !

!AbstractMatrix methodsFor: 'misc. operations' stamp: 'jmv 3/19/2002 20:28'!
norm2Squared
	"(Matrix fromArrayOfArrays: #(
		#(3) 
		#(4)
		#(5)
	)) norm2Squared"

	| result |
	self width > 1 & (self height > 1) ifTrue: [ ^self error: 'by now, only for vectors' ].
	result _ 0.0.
	self width = 1 ifTrue: [
		1 to: self height do: [ :i |
			result _ result + (self atRow: i column: 1) squared ] ]
	ifFalse: [
		1 to: self width do: [ :i |
			result _ result + (self atRow: 1 column: i) squared ] ].
	^result! !

!AbstractMatrix methodsFor: 'misc. operations' stamp: 'jmv 4/10/2002 19:51'!
normalizeColumns
	"Divide each column by its 2-norm"
	"(Matrix fromArrayOfArrays: #(
		#(1 20 ) 
		#(1 4 ) 
	)) normalizeColumns"

	| normSquared norm |
	1 to: width do: [ :j |
		normSquared _ 0.0.
		1 to: height do: [ :i |
			normSquared _ normSquared + (self atRow: i column: j) squared ].
		norm _ normSquared sqrt.
		norm > 0 ifTrue: [
			self multiplyColumn: j by: 1 / norm ] ]! !

!AbstractMatrix methodsFor: 'misc. operations' stamp: 'jmv 4/10/2002 19:52'!
normalizeRows
	"Divide each row by its 2-norm"
	"(Matrix fromArrayOfArrays: #(
		#(1 20 ) 
		#(1 4 ) 
	)) normalizeRows"

	| normSquared norm |
	1 to: height do: [ :i |
		normSquared _ 0.0.
		1 to: width do: [ :j |
			normSquared _ normSquared + (self atRow: i column: j) squared ].
		norm _ normSquared sqrt.
		norm > 0 ifTrue: [
			self multiplyRow: i by: 1 / norm ] ]! !

!AbstractMatrix methodsFor: 'misc. operations' stamp: 'jmv 7/1/2002 13:34'!
orthogonalize
	"Make self an orhonormal matrix, by making columns be linear combinations of the original ones.
	Substract to each column its projection over all the other ones, except for column 1 that remains unchanged
	(except that it is normalized)"

	^self orthogonalizeModifGramSchmidtStartingAtColumn: 1! !

!AbstractMatrix methodsFor: 'misc. operations' stamp: 'jmv 7/2/2002 11:57'!
orthogonalizeModifGramSchmidtStartingAtColumn: jc
	"Make self an orhonormal matrix, by making columns be linear combinations of the original ones.
	Substract to each column its projection over all the other ones, except for column jc that remains unchanged.

	Use a sligthly modified of the Modified Gram-Schmidt method. Instead of starting at column 1, can start on any column.
	(this can be useful to adjust witch columns should be less affected).
	The effect is the same as taking the columns 1..(jc-1), and moving them to the right of the matrix, placing first the jc-1
	column, and last the 1 column; performing the regular Modified Gram-Schmidt method; and placing again each column
	where it belongs. If jc = 1, it's just the regular method.

	Besides, each column is normalized."

	| columnK |
	jc to: self width do: [ :k |

Transcript show: ' ', k printString.
		"Normalize column k"
		columnK _ self columnAt: k.
		columnK normalizeColumns.
		self at: k putColumn: columnK.

		k + 1 to: self width do: [ :j |
			"Substract to column j its projection over column k"
			self at: j putColumn:
				(self columnAt: j) - 
					(columnK *
						(columnK transposed * (self columnAt: j) atRow: 1 column: 1) ) ].

"		1 to: jc-1 do: [ :j |"
		jc-1 to: 1 by: -1 do: [ :j |
			"Substract to column j its projection over column k"
			self at: j putColumn:
				(self columnAt: j) - 
					(columnK *
						(columnK transposed * (self columnAt: j) atRow: 1 column: 1) ) ].
	].

"	1 to: jc-1 do: [ :k |"
	jc-1 to: 1 by: -1 do: [ :k |

Transcript show: ' ', k printString.
		"Normalize column k"
		columnK _ self columnAt: k.
		columnK normalizeColumns.
		self at: k putColumn: columnK.

"		k + 1 to: jc-1 do: [ :j |"
		k - 1 to: 1 by: -1 do: [ :j |
			"Substract to column j its projection over column k"
			self at: j putColumn:
				(self columnAt: j) - 
					(columnK *
						(columnK transposed * (self columnAt: j) atRow: 1 column: 1) ) ].
	].! !

!AbstractMatrix methodsFor: 'misc. operations' stamp: 'jmv 3/19/2002 20:03'!
replaceValues: aBlock
	"Replace each value with the result of evaluating aBlock, with the previous value as the argument"

	1 to: height do: [ :i |
		1 to: width do: [ :j |
			self atRow: i column: j put:
				(aBlock value: (self atRow: i column: j)) ] ]! !

!AbstractMatrix methodsFor: 'misc. operations' stamp: 'jmv 3/19/2002 20:29'!
transpose
	"Traspose the receiver. Modify it."
	"(Matrix fromArrayOfArrays: #(
		#(2 0 0 3) 
		#(1 1 0 0)
		#(1 1 1 0) 
		#(5 1 1 9)
	)) transpose"

	| aux |
	self isSquare ifFalse: [ ^self error: 'Only for Square Matrices' ].
	1 to: self width do: [ :ii |
		ii+1 to: self width do: [ :jj |
			aux _ self atRow: ii column: jj.
			self atRow: ii column: jj put: (self atRow: jj column: ii).
			self atRow: jj column: ii put: (aux) ] ].! !

!AbstractMatrix methodsFor: 'misc. operations' stamp: 'jmv 3/19/2002 20:29'!
transposed
	"Answer a new matrix, transposed."
	"(Matrix fromArrayOfArrays: #(
		#(2 0 0 3) 
		#(1 1 1 0) 
		#(5 1 1 9)
	)) transposed"

	| result |
	result _ Matrix newRows: self width columns: self height.
	1 to: result height do: [ :i |
		1 to: result width do: [ :j |
			result atRow: i column: j put: (self atRow: j column: i) ] ].
	^result! !


!AbstractMatrix methodsFor: 'printing' stamp: 'jmv 5/6/2002 16:16'!
printOn: aStream

	1 to: height do: [ :i |
		aStream nextPutAll: '| '.
		1 to: width do: [ :j |
			aStream print: ((self atRow: i column: j) roundTo: 0.01); nextPut: $ ; nextPut: $ ; nextPut: Character tab ].
		aStream nextPut: $|; cr ]! !


!AbstractMatrix methodsFor: 'squeak forms' stamp: 'jmv 3/22/2002 15:09'!
asForm

	| form val |
	form _ Form extent: self width@self height depth: 32.
	1 to: self height do: [ :i |
		1 to: self width do: [ :j |
			val _ self atRow: i column: j.
			val > 0 ifTrue: [
				form colorAt: (j-1)@(i-1) put: (Color r: val*3 g: 0 b:0) ]
			ifFalse: [
				form colorAt: (j-1)@(i-1) put: (Color r: 0 g: val * (0-3) rounded b:0) ].
		].
	].
	^form
! !


!AbstractMatrix methodsFor: 'private' stamp: 'JMV 11/23/2001 15:07'!
initializeElements
	"Only by subclasses that need it"! !

!AbstractMatrix methodsFor: 'private' stamp: 'JMV 11/23/2001 15:02'!
zero
	"Answer a zero element. If this is a Float matrix it's important to answer a Float zero
	so all arithmethic is Float."

	^elementType == #Float
		ifTrue: [ 0.0 ]
		ifFalse: [ 0 ]! !

"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

AbstractMatrix class
	instanceVariableNames: ''!

!AbstractMatrix class methodsFor: 'instance creation' stamp: 'jmv 2/8/2002 16:13'!
defaultType
	"Could be #Number or #Float"

	^#Float! !

!AbstractMatrix class methodsFor: 'instance creation' stamp: 'JMV 11/23/2001 15:24'!
fromArrayOfArrays: anArray
	"Example:
	Matrix fromArrayOfArrays: #(
		#(1 2 0 0 0 0 3) 
		#(2 4 1 0 0 -4 3) 
		#(0 1 1 1 0 0 3)
		#(0 0 1 1 1 0 3)
		#(0 0 0 1 1 1 3)
		#(0 1 0 0 1 1 3)
	)."

	| result |
	result _ self newRows: anArray size columns: anArray first size.
	result fillWithArrayOfArrays: anArray.
	^result! !

!AbstractMatrix class methodsFor: 'instance creation' stamp: 'JMV 11/23/2001 15:57'!
newColumnVectorSize: size
	"A row vector is just a matrix with height 1"

	^self newRows: size columns: 1! !

!AbstractMatrix class methodsFor: 'instance creation' stamp: 'JMV 11/23/2001 15:56'!
newIdentitySize: size

	^self newIdentityType: self defaultType size: size! !

!AbstractMatrix class methodsFor: 'instance creation' stamp: 'JMV 11/23/2001 15:53'!
newIdentityType: aSymbol size: size

	| result |
	result _ self newType: aSymbol rows: size columns: size.
	result fillDiagonalWithIdentity.
	^result! !

!AbstractMatrix class methodsFor: 'instance creation' stamp: 'jmv 3/19/2002 19:32'!
newRowVectorSize: size
	"A row vector is just a matrix with height 1"

	^self newRows: 1 columns: size! !

!AbstractMatrix class methodsFor: 'instance creation' stamp: 'JMV 11/23/2001 15:54'!
newRows: n columns: m
	"See the comment in #newType:rows:columns:"

	^self newType: self defaultType rows: n columns: m! !

!AbstractMatrix class methodsFor: 'instance creation' stamp: 'JMV 11/23/2001 15:54'!
newSize: aPoint
	"See the comment in #newType:size:"

	^self newType: self defaultType size: aPoint! !

!AbstractMatrix class methodsFor: 'instance creation' stamp: 'JMV 11/23/2001 15:19'!
newType: aSymbol rows: n columns: m
	"aSymbol == #Number   =>  regular representation (with SmallInts, LargeInts, Fractions 
								or Floats as necessary)
	 aSymbol == #Float      =>  special optimized representation (with FloatArrays)"

	^self new initType: aSymbol rows: n columns: m! !

!AbstractMatrix class methodsFor: 'instance creation' stamp: 'JMV 11/23/2001 15:45'!
newType: aSymbol size: aPoint
	"aSymbol == #Number   =>  regular representation (with SmallInts, LargeInts, Fractions 
								or Floats as necessary)
	 aSymbol == #Float      =>  special optimized representation (with FloatArrays)"

	^self new initType: aSymbol rows: aPoint y columns: aPoint x! !

!AbstractMatrix class methodsFor: 'instance creation' stamp: 'JMV 11/23/2001 15:57'!
newVectorSize: size
	"Vectors are by default Column Vectors"

	^self newColumnVectorSize: size! !


AbstractMatrix subclass: #BandMatrix
	instanceVariableNames: 'elements bandWidth '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Numeric Linear Algebra'!
!BandMatrix commentStamp: '<historical>' prior: 0!
My instances are Band Matrices, i.e. square matrices whose elements are zero outside a band around the diagonal. So, only the band is stored.!


!BandMatrix methodsFor: 'accessing' stamp: 'JMV 11/23/2001 15:00'!
atRow: i column: j
	"i and j go from 1 to size (height or width)"

	| position |
	position _ (self elementsIndexForRow: i column: j).
	^ position notNil
		ifTrue: [ elements at: position ]
		ifFalse: [ self zero ]! !

!BandMatrix methodsFor: 'accessing' stamp: 'jmv 7/2/2002 17:51'!
atRow: i column: j ifInBandPut: element
	"i and j go from 1 to size (height or width).
	Do not fail if attemting to store outside band. Just do nothing."

	| position |
	position _ (self elementsIndexForRow: i column: j).
	^ position notNil
		ifTrue: [ elements at: position put: element ]! !

!BandMatrix methodsFor: 'accessing' stamp: 'JMV 11/23/2001 15:22'!
atRow: i column: j put: element
	"i and j go from 1 to size (height or width)"

	| position |
	position _ (self elementsIndexForRow: i column: j).
	^ position notNil
		ifTrue: [ elements at: position put: element ]
		ifFalse: [ self error: 'Can not store outside band' ]! !

!BandMatrix methodsFor: 'accessing' stamp: 'jmv 7/5/2002 14:44'!
bandWidth
	^bandWidth! !


!BandMatrix methodsFor: 'aritmethic' stamp: 'jmv 7/12/2002 17:22'!
* aMatrixOrNumber

	| result |
	(aMatrixOrNumber class = self class and: [ self height = aMatrixOrNumber width ]) ifTrue: [
		"The following line is not necesary. Both matrices are square and of the same size"
		"self width = aMatrixOrNumber height ifFalse: [ ^self error: 'Matrix sizes do not match' ]."
		result _ self class 
			newRowsAndColumns: self height 
			bandWidth: (self bandWidth + aMatrixOrNumber bandWidth - 1 min: 2 * self height - 1).
		self fill: result withResultOfMultiplyBy: aMatrixOrNumber ]
	ifFalse: [ 
		"Warning: This can also calls self fill:withResultOfMultiplyBy:"
		result _ super * aMatrixOrNumber
	].
	^result
	! !

!BandMatrix methodsFor: 'aritmethic' stamp: 'jmv 7/12/2002 17:11'!
fill: resultMatrix withResultOfMultiplyBy: matrixB
	"Fill resultMatrix with the result of self * matrixB"

	^ ((resultMatrix class = matrixB class) and: [ resultMatrix class = self class ])
		ifTrue: [
			"We assume resultMatrix has the correct band size, as prepared in #*"
			self primAllFloatBandMatricesAddTo: resultMatrix elements bw: resultMatrix bandWidth 
				resultOfMultiply: elements h: height bw: bandWidth 
				by: matrixB elements bw: matrixB bandWidth ]
		ifFalse: [ 
			(resultMatrix class = Matrix and: [ 
					resultMatrix class = matrixB class and: [ resultMatrix width = 1 and: [ matrixB width = 1 ] ] ])
				ifTrue: [
					self primFloatVectorAddTo: resultMatrix elements size: resultMatrix height
						resultOfBandMatrixMultiply: self elements bw: self bandWidth
						byVector: matrixB elements ]
				ifFalse: [
					self nonInlinedFill: resultMatrix withResultOfMultiplyBy: matrixB ] ]! !

!BandMatrix methodsFor: 'aritmethic' stamp: 'jmv 7/12/2002 17:11'!
fill: resultMatrix withResultOfTransposedMultiplyBy: matrixB
	"Fill resultMatrix with the result of self transposed * matrixB"

	^ ((resultMatrix class = matrixB class) and: [ resultMatrix class = self class ])
		ifTrue: [
			"We assume resultMatrix has the correct band size, as prepared in #*"
			self primAllFloatBandMatricesAddTo: resultMatrix elements bw: resultMatrix bandWidth 
				resultOfTransposedMultiply: elements h: height bw: bandWidth 
				by: matrixB elements bw: matrixB bandWidth ]
		ifFalse: [ 
			(resultMatrix class = Matrix and: [ 
					resultMatrix class = matrixB class and: [ resultMatrix width = 1 and: [ matrixB width = 1 ] ] ])
				ifTrue: [
					self primFloatVectorAddTo: resultMatrix elements size: resultMatrix height
						resultOfBandMatrixTransposedMultiply: self elements bw: self bandWidth
						byVector: matrixB elements ]
				ifFalse: [
					self error: 'This case is not supported and should be handled in calling method' ] ]! !

!BandMatrix methodsFor: 'aritmethic' stamp: 'jmv 7/5/2002 14:43'!
nonInlinedFill: resultMatrix withResultOfMultiplyBy: aMatrix
	"Fill resultMatrix with the result of self * aMatrix"
	"This non-inlined version is used if resultMatrix or aMatrix is not a band matrix"

	| elem halfBW |
	halfBW _ bandWidth - 1 / 2.
	1 to: resultMatrix height do: [ :i |
		1 to: resultMatrix width do: [ :j |
			elem _ 0.
			(i - halfBW max: 1) to: (i + halfBW min: width) do: [ :k |
				elem _ elem + ((self atRow: i column: k) * (aMatrix atRow: k column: j)) ].
			resultMatrix atRow: i column: j put: elem ] ]! !

!BandMatrix methodsFor: 'aritmethic' stamp: 'jmv 7/15/2002 12:42'!
transposedMultiplyBy: aMatrixOrNumber
	"Answers the same as self transposed * aMatrixOrNumber, but saves time and space,
	avoiding to build self transposed, i aMatrixOrNumber is a float vector matrix."

	| result |
	(aMatrixOrNumber class = self class and: [ self height = aMatrixOrNumber width ])
		ifTrue: [
			"The following line is not necesary. Both matrices are square and of the same size"
			"self width = aMatrixOrNumber height ifFalse: [ ^self error: 'Matrix sizes do not match' ]."
			result _ self class 
				newRowsAndColumns: self width 
				bandWidth: (self bandWidth + aMatrixOrNumber bandWidth - 1 min: 2 * self width - 1).
			self fill: result withResultOfTransposedMultiplyBy: aMatrixOrNumber ]
		ifFalse: [ 
			(aMatrixOrNumber class = Matrix and: [
				aMatrixOrNumber width = 1 and: [
					self height = aMatrixOrNumber height ]]) ifTrue: [
				result _ Matrix newVectorSize: self width.
				self fill: result withResultOfTransposedMultiplyBy: aMatrixOrNumber ]
			ifFalse: [ result _ self transposed * aMatrixOrNumber ] ].
	^result
	! !


!BandMatrix methodsFor: 'initialization' stamp: 'JMV 11/23/2001 15:13'!
initType: aSymbol rows: n columns: m

	n = m ifFalse: [ ^self error: 'Band Matrices must be square' ].
	^super initType: aSymbol rows: n columns: m! !

!BandMatrix methodsFor: 'initialization' stamp: 'jmv 4/10/2002 16:32'!
initType: aSymbol rowsAndColumns: n bandWidth: bw

	elementType _ aSymbol.
	height _ n.
	width _ n.
	bandWidth _ bw.
	self initializeElements.! !


!BandMatrix methodsFor: 'private' stamp: 'jmv 7/5/2002 14:44'!
elements
	^elements! !

!BandMatrix methodsFor: 'private' stamp: 'jmv 7/2/2002 17:10'!
elementsIndexForRow: i column: j
	"Band matrices use this special representation.
	Answer nil if outside the band.
	For reducing OS's virtual memory page faults when doing column
	operations on huge matrices, the elements of each column are stored together."

	(i between: 1 and: height) & (j between: 1 and: width)
		ifFalse: [ self error: 'Attempt to access outside bounds' ].
	^ (j-i) abs * 2 < bandWidth
		ifTrue: [ (j-1)*(bandWidth-1) + i ]
		ifFalse: [ nil ]! !

!BandMatrix methodsFor: 'private' stamp: 'jmv 7/5/2002 15:02'!
initializeElements

	"Make me real wide if band width is not specified"
	bandWidth isNil ifTrue: [ bandWidth _ height * 2 - 1 ].
	elementType == #Float
		ifTrue: [ elements _ FloatArray new: height * bandWidth ]
		ifFalse: [
			elements _ Array new: height * bandWidth.
			1 to: elements size do: [ :i |
				elements at: i put: 0 ] ]! !


!BandMatrix methodsFor: 'misc. operations' stamp: 'jmv 7/3/2002 17:51'!
copyWithBandWidth: bw

	| result |
	result _ BandMatrix newRowsAndColumns: self width bandWidth: bw.
	result fillBandWith: self.
	^ result! !

!BandMatrix methodsFor: 'misc. operations' stamp: 'jmv 4/10/2002 17:15'!
fillBandWith: aMatrix

	| halfBW |
	halfBW _ bandWidth - 1 / 2.
	1 to: self height do: [ :i |
		(i - halfBW max: 1) to: (i + halfBW min: width) do: [ :j |
			self atRow: i column: j put: (aMatrix atRow: i column: j) ] ]! !

!BandMatrix methodsFor: 'misc. operations' stamp: 'jmv 7/2/2002 18:00'!
multiplyColumn: j by: factor

	| halfBW |
	halfBW _ bandWidth - 1 / 2.
	(j - halfBW max: 1) to: (j + halfBW min: height) do: [ :i |
		self atRow: i column: j put: (self atRow: i column: j) * factor ]! !

!BandMatrix methodsFor: 'misc. operations' stamp: 'jmv 7/2/2002 17:59'!
normalizeColumns
	"Divide each column by its 2-norm"
	"(Matrix fromArrayOfArrays: #(
		#(1 20 ) 
		#(1 4 ) 
	)) normalizeColumns"

	| normSquared norm halfBW |
	halfBW _ bandWidth - 1 / 2.
	1 to: width do: [ :j |
		normSquared _ 0.0.
		(j - halfBW max: 1) to: (j + halfBW min: height) do: [ :i |
			normSquared _ normSquared + (self atRow: i column: j) squared ].
		norm _ normSquared sqrt.
		norm > 0 ifTrue: [
			self multiplyColumn: j by: 1 / norm ] ]! !

!BandMatrix methodsFor: 'misc. operations' stamp: 'jmv 7/3/2002 19:25'!
orthogonalizeModifGramSchmidtStartingAtColumn: jc
	"Make self an orhonormal matrix, by making columns be linear combinations of the original ones.
	Substract to each column its projection over all the other ones, except for column jc that remains unchanged.

	Use a sligthly modified of the Modified Gram-Schmidt method. Instead of starting at column 1, can start on any column.
	(this can be useful to adjust witch columns should be less affected).
	The effect is the same as taking the columns 1..(jc-1), and moving them to the right of the matrix, placing first the jc-1
	column, and last the 1 column; performing the regular Modified Gram-Schmidt method; and placing again each column
	where it belongs. If jc = 1, it's just the regular method.

	Besides, each column is normalized.

	Based on the code in AbstractMatrix, but hand inlined and hand optimized"

	^self primFloatBandMatrixOrthogonalizeModifGramSchmidtStartingAtCol: jc
		mxH: height mxW: width mxElem: elements mxBW: bandWidth! !

!BandMatrix methodsFor: 'misc. operations' stamp: 'jmv 7/5/2002 13:19'!
regularOrthogonalizeModifGramSchmidtStartingAtColumn: jc
	"jmv - This method should be erased after debugging the plugin version"
	"Make self an orhonormal matrix, by making columns be linear combinations of the original ones.
	Substract to each column its projection over all the other ones, except for column jc that remains unchanged.

	Use a sligthly modified of the Modified Gram-Schmidt method. Instead of starting at column 1, can start on any column.
	(this can be useful to adjust witch columns should be less affected).
	The effect is the same as taking the columns 1..(jc-1), and moving them to the right of the matrix, placing first the jc-1
	column, and last the 1 column; performing the regular Modified Gram-Schmidt method; and placing again each column
	where it belongs. If jc = 1, it's just the regular method.

	Besides, each column is normalized.

	Based on the code in AbstractMatrix, but hand inlined and hand optimized"

	| halfBW kLessOneTimesBandWidthLessOne colKTop colKBottom normSquared norm colKIdx jLessOneTimesBandWidthLessOne 
		colJTop colJBottom intersectionColsKJTop intersectionColsKJBottom correlationFactor threshold thisJThreshold colJIdx colJMaxAbsValue |

	threshold _ 0.05.
	halfBW _ bandWidth - 1 / 2.
	jc to: self width do: [ :k |
		Transcript show: ' ', k printString.
		"Normalize column k"
		kLessOneTimesBandWidthLessOne _ (k-1) * (bandWidth-1).
		colKTop _ k - halfBW max: 1.
		colKBottom _ k + halfBW min: height.
		normSquared _ 0.0.
		colKTop to: colKBottom do: [ :i |
			normSquared _ normSquared + (elements at: kLessOneTimesBandWidthLessOne + i) squared ].
		norm _ normSquared sqrt.
		norm > 0 ifTrue: [
			colKTop to: colKBottom do: [ :i |
				colKIdx _ kLessOneTimesBandWidthLessOne + i.
				elements at: colKIdx put: (elements at: colKIdx) / norm ] ].

		"Substract column k to all those at it's right"
		k + 1 to: self width do: [ :j |
			"Substract to column j its projection over column k"
			jLessOneTimesBandWidthLessOne _ (j-1) * (bandWidth-1).
			colJTop _ j - halfBW max: 1.
			colJBottom _ j + halfBW min: height.
			intersectionColsKJTop _ colKTop max: colJTop.
			intersectionColsKJBottom _ colKBottom min: colJBottom.
			"Find Correlation Factor"
			colJMaxAbsValue _ 0.
			colJTop to: colJBottom do: [ :i |
				colJMaxAbsValue _ colJMaxAbsValue max: (elements at: jLessOneTimesBandWidthLessOne + i) abs ].
			correlationFactor _ 0.
			intersectionColsKJTop to: intersectionColsKJBottom do: [ :i |
				correlationFactor _ correlationFactor + 
					((elements at: kLessOneTimesBandWidthLessOne + i) * (elements at: jLessOneTimesBandWidthLessOne + i)) ].
			correlationFactor abs > 0.001 ifTrue: [
				thisJThreshold _ threshold * colJMaxAbsValue / correlationFactor abs.
				"The intersection between the height of both columns, substract"
				intersectionColsKJTop to: intersectionColsKJBottom do: [ :i |
					colJIdx _ jLessOneTimesBandWidthLessOne + i.
					elements at: colJIdx put: (elements at: colJIdx) - ((elements at: kLessOneTimesBandWidthLessOne + i) * correlationFactor) ].
				"Top part of column K, that would fall off the band if substracted to column J"
				colKTop to: (colJTop-1 min: colKBottom) do: [ :i |
					(elements at: kLessOneTimesBandWidthLessOne + i) abs > thisJThreshold ifTrue: [ self error: 'Need bigger band!!' ] ].
				"Bottom part of column K, that would fall off the band if substracted to column J"
				(colJBottom+1 max: colKTop) to: colKBottom do: [:i |
					(elements at: kLessOneTimesBandWidthLessOne + i) abs > thisJThreshold ifTrue: [ self error: 'Need bigger band!!' ] ].
			].
		].

		jc-1 to: 1 by: -1 do: [ :j |
			"Substract to column j its projection over column k"
			jLessOneTimesBandWidthLessOne _ (j-1) * (bandWidth-1).
			colJTop _ j - halfBW max: 1.
			colJBottom _ j + halfBW min: height.
			intersectionColsKJTop _ colKTop max: colJTop.
			intersectionColsKJBottom _ colKBottom min: colJBottom.
			"Find Correlation Factor"
			colJMaxAbsValue _ 0.
			colJTop to: colJBottom do: [ :i |
				colJMaxAbsValue _ colJMaxAbsValue max: (elements at: jLessOneTimesBandWidthLessOne + i) abs ].
			correlationFactor _ 0.
			intersectionColsKJTop to: intersectionColsKJBottom do: [ :i |
				correlationFactor _ correlationFactor + 
					((elements at: kLessOneTimesBandWidthLessOne + i) * (elements at: jLessOneTimesBandWidthLessOne + i)) ].
			correlationFactor abs > 0.001 ifTrue: [
				thisJThreshold _ threshold * colJMaxAbsValue / correlationFactor abs.
				"The intersection between the height of both columns, substract"
				intersectionColsKJTop to: intersectionColsKJBottom do: [ :i |
					colJIdx _ jLessOneTimesBandWidthLessOne + i.
					elements at: colJIdx put: (elements at: colJIdx) - ((elements at: kLessOneTimesBandWidthLessOne + i) * correlationFactor) ].
				"Top part of column K, that would fall off the band if substracted to column J"
				colKTop to: (colJTop-1 min: colKBottom) do: [ :i |
					(elements at: kLessOneTimesBandWidthLessOne + i) abs > thisJThreshold ifTrue: [ self error: 'Need bigger band!!' ] ].
				"Bottom part of column K, that would fall off the band if substracted to column J"
				(colJBottom+1 max: colKTop) to: colKBottom do: [:i |
					(elements at: kLessOneTimesBandWidthLessOne + i) abs > thisJThreshold ifTrue: [ self error: 'Need bigger band!!' ] ].
			].
		].
	].

	jc-1 to: 1 by: -1 do: [ :k |
		Transcript show: ' ', k printString.
		"Normalize column k"
		kLessOneTimesBandWidthLessOne _ (k-1) * (bandWidth-1).
		colKTop _ k - halfBW max: 1.
		colKBottom _ k + halfBW min: height.
		normSquared _ 0.0.
		colKTop to: colKBottom do: [ :i |
			normSquared _ normSquared + (elements at: kLessOneTimesBandWidthLessOne + i) squared ].
		norm _ normSquared sqrt.
		norm > 0 ifTrue: [
			colKTop to: colKBottom do: [ :i |
				colKIdx _ kLessOneTimesBandWidthLessOne + i.
				elements at: colKIdx put: (elements at: colKIdx) / norm ] ].

		k - 1 to: 1 by: -1 do: [ :j |
			"Substract to column j its projection over column k"
			jLessOneTimesBandWidthLessOne _ (j-1) * (bandWidth-1).
			colJTop _ j - halfBW max: 1.
			colJBottom _ j + halfBW min: height.
			intersectionColsKJTop _ colKTop max: colJTop.
			intersectionColsKJBottom _ colKBottom min: colJBottom.
			"Find Correlation Factor"
			colJMaxAbsValue _ 0.
			colJTop to: colJBottom do: [ :i |
				colJMaxAbsValue _ colJMaxAbsValue max: (elements at: jLessOneTimesBandWidthLessOne + i) abs ].
			correlationFactor _ 0.
			intersectionColsKJTop to: intersectionColsKJBottom do: [ :i |
				correlationFactor _ correlationFactor + 
					((elements at: kLessOneTimesBandWidthLessOne + i) * (elements at: jLessOneTimesBandWidthLessOne + i)) ].
			correlationFactor abs > 0.001 ifTrue: [
				thisJThreshold _ threshold * colJMaxAbsValue / correlationFactor abs.
				"The intersection between the height of both columns, substract"
				intersectionColsKJTop to: intersectionColsKJBottom do: [ :i |
					colJIdx _ jLessOneTimesBandWidthLessOne + i.
					elements at: colJIdx put: (elements at: colJIdx) - ((elements at: kLessOneTimesBandWidthLessOne + i) * correlationFactor) ].
				"Top part of column K, that would fall off the band if substracted to column J"
				colKTop to: (colJTop-1 min: colKBottom) do: [ :i |
					(elements at: kLessOneTimesBandWidthLessOne + i) abs > thisJThreshold ifTrue: [ self error: 'Need bigger band!!' ] ].
				"Bottom part of column K, that would fall off the band if substracted to column J"
				(colJBottom+1 max: colKTop) to: colKBottom do: [:i |
					(elements at: kLessOneTimesBandWidthLessOne + i) abs > thisJThreshold ifTrue: [ self error: 'Need bigger band!!' ] ].
			].
		].
	].
! !

!BandMatrix methodsFor: 'misc. operations' stamp: 'jmv 7/12/2002 17:24'!
replaceValues: aBlock
	"Replace each value with the result of evaluating aBlock, with the previous value as the argument"

	1 to: elements size do: [ :i |
		elements at: i put: (aBlock value: (elements at: i)) ]! !

!BandMatrix methodsFor: 'misc. operations' stamp: 'jmv 7/5/2002 15:40'!
transposed
	"Answer a new matrix, transposed."
	"(BandMatrix fromArrayOfArrays: #(
		#(2 0 0 3) 
		#(1 1 1 0) 
		#(5 1 1 9)
		#(5 1 1 9)
	)) transposed"

	| result halfBW |
	halfBW _ bandWidth - 1 / 2.
	result _ self class newRowsAndColumns: self width bandWidth: self bandWidth.
	1 to: result height do: [ :i |
		(i - halfBW max: 1) to: (i + halfBW min: result width) do: [ :j |
			result atRow: i column: j put: (self atRow: j column: i) ] ].
	^result! !


!BandMatrix methodsFor: 'plugin primitives' stamp: 'jmv 7/5/2002 15:09'!
primAllFloatBandMatricesAddTo: resultMatrixElements bw: resultMatrixBW resultOfMultiply: matrixAElements h: matrixAHeight bw: matrixABW by: matrixBElements bw: matrixBBW

	<primitive: 'primAllFloatBandMatricesAddTobwresultOfMultiplyhbwbybw' module: 'NumericLinealAlgebraPlugin'>
	^NumericLinealAlgebraPlugin
		doPrimitive: 'primAllFloatBandMatricesAddTo:bw:resultOfMultiply:h:bw:by:bw:'
		withArguments: {resultMatrixElements. resultMatrixBW. matrixAElements. matrixAHeight. 
			matrixABW. matrixBElements. matrixBBW}! !

!BandMatrix methodsFor: 'plugin primitives' stamp: 'jmv 7/10/2002 10:25'!
primAllFloatBandMatricesAddTo: resultMatrixElements bw: resultMatrixBW resultOfTransposedMultiply: matrixAElements h: matrixAHeight bw: matrixABW by: matrixBElements bw: matrixBBW

	<primitive: 'primAllFloatBandMatricesAddTobwresultOfTransposedMultiplyhbwbybw' module: 'NumericLinealAlgebraPlugin'>
	^NumericLinealAlgebraPlugin
		doPrimitive: 'primAllFloatBandMatricesAddTo:bw:resultOfTransposedMultiply:h:bw:by:bw:'
		withArguments: {resultMatrixElements. resultMatrixBW. matrixAElements. matrixAHeight. 
			matrixABW. matrixBElements. matrixBBW}! !

!BandMatrix methodsFor: 'plugin primitives' stamp: 'jmv 7/3/2002 19:24'!
primFloatBandMatrixOrthogonalizeModifGramSchmidtStartingAtCol: jc mxH: h mxW: w mxElem: e mxBW: bw

	<primitive: 'primFloatBandMatrixOrthogonalizeModifGramSchmidtStartingAtCol' module: 'NumericLinealAlgebraPlugin'>
	^NumericLinealAlgebraPlugin
		doPrimitive: 'primFloatBandMatrixOrthogonalizeModifGramSchmidtStartingAtCol:mxH:mxW:mxElem:mxBW:'
		withArguments: {jc. h. w. e. bw}! !

!BandMatrix methodsFor: 'plugin primitives' stamp: 'jmv 7/8/2002 14:48'!
primFloatVectorAddTo: resultMatrixElements size: resultMatrixSize resultOfBandMatrixMultiply: matrixAElements bw: matrixABW byVector: matrixBElements

	<primitive: 'primFloatVectorAddTosizeresultOfBandMatrixMultiplybwbyVector' module: 'NumericLinealAlgebraPlugin'>
	^NumericLinealAlgebraPlugin
		doPrimitive: 'primFloatVectorAddTo:size:resultOfBandMatrixMultiply:bw:byVector:'
		withArguments: {resultMatrixElements. resultMatrixSize. matrixAElements. matrixABW. matrixBElements}! !

!BandMatrix methodsFor: 'plugin primitives' stamp: 'jmv 7/8/2002 13:35'!
primFloatVectorAddTo: resultMatrixElements size: resultMatrixSize resultOfBandMatrixTransposedMultiply: matrixAElements bw: matrixABW byVector: matrixBElements

	<primitive: 'primFloatVectorAddTosizeresultOfBandMatrixTransposedMultiplybwbyVector' module: 'NumericLinealAlgebraPlugin'>
	^NumericLinealAlgebraPlugin
		doPrimitive: 'primFloatVectorAddTo:size:resultOfBandMatrixTransposedMultiply:bw:byVector:'
		withArguments: {resultMatrixElements. resultMatrixSize. matrixAElements. matrixABW. matrixBElements}! !

"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

BandMatrix class
	instanceVariableNames: ''!

!BandMatrix class methodsFor: 'as yet unclassified' stamp: 'jmv 4/10/2002 16:35'!
newRowsAndColumns: n bandWidth: bw
	"See the comment in #newType:rowsAndColumns:bandWidth:"

	^self newType: self defaultType rowsAndColumns: n bandWidth: bw! !

!BandMatrix class methodsFor: 'as yet unclassified' stamp: 'jmv 7/3/2002 17:52'!
newType: aSymbol rowsAndColumns: n bandWidth: bw
	"aSymbol == #Number   =>  regular representation (with SmallInts, LargeInts, Fractions 
								or Floats as necessary)
	 aSymbol == #Float      =>  special optimized representation (with FloatArrays)"

	bw odd ifFalse: [ ^self error: 'Banded Matrices must have odd band width'].
	bw >= (2*n) ifTrue: [ ^self error: 'Band Width can not be so large' ].
	^self new initType: aSymbol rowsAndColumns: n bandWidth: bw! !


AbstractMatrix subclass: #Matrix
	instanceVariableNames: 'elements '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Numeric Linear Algebra'!
!Matrix commentStamp: '<historical>' prior: 0!
My instances are "full" rectangular Matrices. The elements are stored in a kind of Array.!


!Matrix methodsFor: 'accessing' stamp: 'JMV 11/23/2001 14:58'!
atRow: i column: j
	"i and j go from 1 to size (height or width)"

	^ elements at: (self elementsIndexForRow: i column: j)! !

!Matrix methodsFor: 'accessing' stamp: 'JMV 11/23/2001 15:22'!
atRow: i column: j put: element
	"i and j go from 1 to size (height or width)"

	^ elements at: (self elementsIndexForRow: i column: j) put: element! !

!Matrix methodsFor: 'accessing' stamp: 'jmv 3/19/2002 20:16'!
max

	^elements max! !


!Matrix methodsFor: 'misc. operations' stamp: 'jmv 7/12/2002 16:51'!
orthogonalizeModifGramSchmidtStartingAtColumn: jc
	"Make self an orhonormal matrix, by making columns be linear combinations of the original ones.
	Substract to each column its projection over all the other ones, except for column jc that remains unchanged.

	Use a sligthly modified of the Modified Gram-Schmidt method. Instead of starting at column 1, can start on any column.
	(this can be useful to adjust witch columns should be less affected).
	The effect is the same as taking the columns 1..(jc-1), and moving them to the right of the matrix, placing first the jc-1
	column, and last the 1 column; performing the regular Modified Gram-Schmidt method; and placing again each column
	where it belongs. If jc = 1, it's just the regular method.

	Besides, each column is normalized.

	Based on the code in AbstractMatrix, but hand inlined and hand optimized"

	^self primFloatMatrixOrthogonalizeModifGramSchmidtStartingAtCol: jc
		mxH: height mxW: width mxElem: elements! !

!Matrix methodsFor: 'misc. operations' stamp: 'jmv 7/12/2002 17:24'!
replaceValues: aBlock
	"Replace each value with the result of evaluating aBlock, with the previous value as the argument"

	1 to: elements size do: [ :i |
		elements at: i put: (aBlock value: (elements at: i)) ]! !


!Matrix methodsFor: 'aritmethic' stamp: 'jmv 7/12/2002 17:55'!
fill: resultMatrix withResultOfMultiplyBy: aMatrix
	"Fill resultMatrix with the result of self * aMatrix"
	"This reimplementation is just for speed, and for speed's sake it violates encapsulation"

	| elem aMatrixElements resultElements aMatrixHeight resultHeight |
	(aMatrix class == self class) & (resultMatrix class == self class) ifFalse: [ 
		^ super fill: resultMatrix withResultOfMultiplyBy: aMatrix ].

	(resultMatrix width = 1 and: [ aMatrix width = 1 ])
		ifTrue: [
			self primFloatVectorAddTo: resultMatrix elements size: resultMatrix height
				resultOfMatrixMultiply: self elements
				byVector: aMatrix elements size: aMatrix height ]
		ifFalse: [
			aMatrixElements _ aMatrix elements.
			aMatrixHeight _ aMatrix height.
			resultElements _ resultMatrix elements.
			resultHeight _ resultMatrix height.
			1 to: resultMatrix height do: [ :i |
				1 to: resultMatrix width do: [ :j |
					elem _ 0.
					1 to: self width do: [ :k |
						elem _ elem + (
							(elements at: (k-1*height+i)) *
							(aMatrixElements at: (j-1*aMatrixHeight+k))
						)
					].
					resultElements at: (j-1*resultHeight+i) put: elem ] ] ]! !

!Matrix methodsFor: 'aritmethic' stamp: 'jmv 7/12/2002 17:42'!
fill: resultMatrix withResultOfTransposedMultiplyBy: matrixB
	"Fill resultMatrix with the result of self transposed * matrixB"

	^ (resultMatrix class = self class and: [ 
			resultMatrix class = matrixB class and: [ resultMatrix width = 1 and: [ matrixB width = 1 ] ] ])
		ifTrue: [
			self primFloatVectorAddTo: resultMatrix elements size: resultMatrix height
				resultOfMatrixTransposedMultiply: self elements
				byVector: matrixB elements size: matrixB height ]
		ifFalse: [
			self error: 'This case is not supported and should be handled in calling method' ]! !

!Matrix methodsFor: 'aritmethic' stamp: 'jmv 7/15/2002 12:40'!
transposedMultiplyBy: aMatrixOrNumber
	"Answers the same as self transposed * aMatrixOrNumber, but saves time and space,
	avoiding to build self transposed, i aMatrixOrNumber is a float vector matrix."

	| result |
	(aMatrixOrNumber class = Matrix and: [
		aMatrixOrNumber width = 1 and: [
			self height = aMatrixOrNumber height ]]) ifTrue: [
		result _ Matrix newVectorSize: self width.
		self fill: result withResultOfTransposedMultiplyBy: aMatrixOrNumber ]
	ifFalse: [ result _ self transposed * aMatrixOrNumber ].
	^result
	! !


!Matrix methodsFor: 'private' stamp: 'jmv 4/17/2002 12:10'!
elements
	"Warning: Violates encapsulation. Use with care."

	^elements! !

!Matrix methodsFor: 'private' stamp: 'jmv 7/12/2002 17:52'!
elementsIndexForRow: i column: j
	"i and j go from 1 to size (height or width)"

	(i between: 1 and: height) & (j between: 1 and: width)
		ifFalse: [ self error: 'Attempt to access outside bounds' ].
	^ j-1*height+i! !

!Matrix methodsFor: 'private' stamp: 'JMV 11/23/2001 15:11'!
initializeElements

	elementType == #Float
		ifTrue: [ elements _ FloatArray new: height * width ]
		ifFalse: [
			elements _ Array new: height * width.
			1 to: elements size do: [ :i |
				elements at: i put: 0 ] ]! !


!Matrix methodsFor: 'linear systems operations' stamp: 'jmv 7/12/2002 17:59'!
substractRow: i multipliedBy: factor to: k startingAtColumn: jStart
	"This reimplementation is just for speed, and for speed's sake it violates encapsulation"

	| jLessOneTimesHeight |
	jStart to: width do: [ :j |
		jLessOneTimesHeight _ j-1*height.
		elements at: (jLessOneTimesHeight+k) put:
			(elements at: (jLessOneTimesHeight+k)) - ((elements at: (jLessOneTimesHeight+i)) * factor) ]! !

!Matrix methodsFor: 'linear systems operations' stamp: 'jmv 7/12/2002 17:56'!
triangleWithPartialPivoting
	"Triangle self, and answer the row permutation count"
	"This reimplementation is just for speed, and for speed's sake it violates encapsulation"
	"There are still some acesses with #atRow:column: that could also be hand inlined"
	"(Matrix fromArrayOfArrays: #(
		#(1 2 0 0 0 0 3) 
		#(2 4 1 0 0 -4 3) 
		#(0 1 1 1 0 0 3)
		#(0 0 1 1 1 0 3)
		#(0 0 0 1 1 1 3)
		#(0 1 0 0 1 1 3)
	)) triangleWithPartialPivoting; yourself"

	| j jdelta k factor permutationCount jLessOneTimesHeight pivot |
	permutationCount _ 0.
	jdelta _ 0.
	1 to: self height-1 do: [ :i |

		[
			j _ i + jdelta.
			j <= self width ifFalse: [ ^self ].
 			"Look for the pivot for column j, from row i down. Call it row k"
			k _ self rowWithMaxInColumn: j startingAtRow: i.
			((self atRow: k column: j) = 0) & (j < self width) ] whileTrue: [ jdelta _ jdelta + 1 ].

		"Permute rows k and i"
		i = k ifFalse: [
			self permuteRow: i withRow: k.
			permutationCount _ permutationCount + 1 ].

      	"Substract the row i to all the ones below it"
		jLessOneTimesHeight _ j-1*height.
		pivot _ elements at: (jLessOneTimesHeight+i).
		i+1 to: self height do: [ :ii |
			(self atRow: ii column: j) = 0.0 ifFalse: [
				factor _ (elements at: (jLessOneTimesHeight+ii)) / pivot.
				"Only after column j"
				elements at: (jLessOneTimesHeight+ii) put: 0.
				self substractRow: i multipliedBy: factor to: ii startingAtColumn: j+1 ] ]
	].
	^ permutationCount! !


!Matrix methodsFor: 'plugin primitives' stamp: 'jmv 7/12/2002 17:41'!
primFloatMatrixOrthogonalizeModifGramSchmidtStartingAtCol: jc mxH: h mxW: w mxElem: e

	<primitive: 'primFloatMatrixOrthogonalizeModifGramSchmidtStartingAtCol' module: 'NumericLinealAlgebraPlugin'>
	^NumericLinealAlgebraPlugin
		doPrimitive: 'primFloatMatrixOrthogonalizeModifGramSchmidtStartingAtCol:mxH:mxW:mxElem:'
		withArguments: {jc. h. w. e}! !

!Matrix methodsFor: 'plugin primitives' stamp: 'jmv 7/12/2002 17:07'!
primFloatVectorAddTo: resultMatrixElements size: resultMatrixSize resultOfMatrixMultiply: matrixAElements byVector: matrixBElements size: matrixBSize

	<primitive: 'primFloatVectorAddTosizeresultOfMatrixMultiplybyVectorsize' module: 'NumericLinealAlgebraPlugin'>
	^NumericLinealAlgebraPlugin
		doPrimitive: 'primFloatVectorAddTo:size:resultOfMatrixMultiply:byVector:size:'
		withArguments: {resultMatrixElements. resultMatrixSize. matrixAElements. matrixBElements. matrixBSize}! !

!Matrix methodsFor: 'plugin primitives' stamp: 'jmv 7/12/2002 17:07'!
primFloatVectorAddTo: resultMatrixElements size: resultMatrixSize resultOfMatrixTransposedMultiply: matrixAElements byVector: matrixBElements size: matrixBSize

	<primitive: 'primFloatVectorAddTosizeresultOfMatrixTransposedMultiplybyVectorsize' module: 'NumericLinealAlgebraPlugin'>
	^NumericLinealAlgebraPlugin
		doPrimitive: 'primFloatVectorAddTo:size:resultOfMatrixTransposedMultiply:byVector:size:'
		withArguments: {resultMatrixElements. resultMatrixSize. matrixAElements. matrixBElements. matrixBSize}! !


TestInterpreterPlugin subclass: #NumericLinealAlgebraPlugin
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Numeric Linear Algebra'!
!NumericLinealAlgebraPlugin commentStamp: 'jmv 7/3/2002 18:30' prior: 0!
This plugin includes optimized versions of code for Numeric Lineal Algebra!


!NumericLinealAlgebraPlugin methodsFor: 'initialize-release' stamp: 'jmv 7/3/2002 18:21'!
initialiseModule

	self export: true! !

!NumericLinealAlgebraPlugin methodsFor: 'initialize-release' stamp: 'jmv 7/3/2002 18:21'!
shutdownModule

	self export: true! !


!NumericLinealAlgebraPlugin methodsFor: 'primitives' stamp: 'jmv 7/10/2002 10:22'!
primAllFloatBandMatricesAddTo: resultMatrixElements bw: resultMatrixBW resultOfMultiply: matrixAElements h: matrixAHeight bw: matrixABW by: matrixBElements bw: matrixBBW

	"Add to resultMatrix the result of matrixA * matrixB
		resultMatrix += matrixA * matrixB"
	"Hand inlined and hand optimized. The order of the iterations was choosed trying to minimize
		page faults in a virtual memory system when using large matrices."
	"Warning: if the band in the result matrix is not wide enough,
		ALL THE ELEMENTS OUSIDE THE BAND ARE SIMPLY DISCARDED
		(The correct band width is prepared in BandMatrix>>*)"

	"resultMatrix should contain zeros before calling.
	For safety, matrix sizes should be got from the objects themselves, and the FloatArrays should
	be checked to be big enough"

	| resultMatrixHalfBW matrixAHalfBW matrixBHalfBW resultMatrixBWLessOne matrixABWLessOne matrixBBWLessOne |

	self export: true.
	self
		primitive: 'primAllFloatBandMatricesAddTobwresultOfMultiplyhbwbybw'
		parameters: #(FloatArray SmallInteger FloatArray SmallInteger SmallInteger FloatArray SmallInteger)
		receiver:	#Oop.

	resultMatrixBWLessOne _ resultMatrixBW - 1.
	matrixABWLessOne _ matrixABW - 1.
	matrixBBWLessOne _ matrixBBW - 1.
	resultMatrixHalfBW _ resultMatrixBW - 1 / 2.
	matrixAHalfBW _ matrixABW - 1 / 2.
	matrixBHalfBW _ matrixBBW - 1 / 2.
	"Matrix a height and width are equal, all 3 matrices are square (and band)"
	"Iterate ij over matrixA columns and matrixB rows"
	1-1 to: matrixAHeight-1 do: [:ijLessOne |

		"iterate the j's (columns) for row ij of matrixB (only inside the band) 
			(matrixBWidth = matrixAHeight for the result to be square and band).
			j is also the column of resultMatrix"
		(ijLessOne-matrixBHalfBW max: 1-1) to: (ijLessOne+matrixBHalfBW min: matrixAHeight-1) do: [ :jLessOne |

			"iterate the i's (rows) for column ij of matrixA and column j of resultMatrix
				(only inside both bands)"
			(ijLessOne-(matrixAHalfBW min: resultMatrixHalfBW) max: 1-1) to:
			(ijLessOne+(matrixAHalfBW max: resultMatrixHalfBW) min: matrixAHeight-1) do: [ :iLessOne |

				resultMatrixElements at: (jLessOne * resultMatrixBWLessOne) + iLessOne put:
					(resultMatrixElements at: (jLessOne * resultMatrixBWLessOne) + iLessOne) +
						((matrixAElements at: (ijLessOne * matrixABWLessOne) + iLessOne) *
							(matrixBElements at: (jLessOne * matrixBBWLessOne) + ijLessOne))
			]
		]
	]! !

!NumericLinealAlgebraPlugin methodsFor: 'primitives' stamp: 'jmv 7/10/2002 10:24'!
primAllFloatBandMatricesAddTo: resultMatrixElements bw: resultMatrixBW resultOfTransposedMultiply: matrixAElements h: matrixAHeight bw: matrixABW by: matrixBElements bw: matrixBBW

	"Add to resultMatrix the result of matrixA transposed * matrixB
		resultMatrix += matrixA transposed * matrixB"
	"Hand inlined and hand optimized."
	"Warning: if the band in the result matrix is not wide enough,
		ALL THE ELEMENTS OUSIDE THE BAND ARE SIMPLY DISCARDED
		(The correct band width is prepared in BandMatrix>>transposedMultiplyBy:)"

	"resultMatrix should contain zeros before calling.
	For safety, matrix sizes should be got from the objects themselves, and the FloatArrays should
	be checked to be big enough"

	| resultMatrixHalfBW matrixAHalfBW matrixBHalfBW resultMatrixBWLessOne matrixABWLessOne matrixBBWLessOne |

	self export: true.
	self
		primitive: 'primAllFloatBandMatricesAddTobwresultOfTransposedMultiplyhbwbybw'
		parameters: #(FloatArray SmallInteger FloatArray SmallInteger SmallInteger FloatArray SmallInteger)
		receiver:	#Oop.

	resultMatrixBWLessOne _ resultMatrixBW - 1.
	matrixABWLessOne _ matrixABW - 1.
	matrixBBWLessOne _ matrixBBW - 1.
	resultMatrixHalfBW _ resultMatrixBW - 1 / 2.
	matrixAHalfBW _ matrixABW - 1 / 2.
	matrixBHalfBW _ matrixBBW - 1 / 2.
	"Matrix a height and width are equal, all 3 matrices are square (and band)"
	"Iterate ij over matrixA columns and matrixB rows"
	1-1 to: matrixAHeight-1 do: [:ijLessOne |

		"iterate the j's (columns) for row ij of matrixB (only inside the band) 
			(matrixBWidth = matrixAHeight for the result to be square and band).
			j is also the column of resultMatrix"
		(ijLessOne-matrixBHalfBW max: 1-1) to: (ijLessOne+matrixBHalfBW min: matrixAHeight-1) do: [ :jLessOne |

			"iterate the i's (rows) for column ij of matrixA and column j of resultMatrix
				(only inside both bands)"
			(ijLessOne-(matrixAHalfBW min: resultMatrixHalfBW) max: 1-1) to:
			(ijLessOne+(matrixAHalfBW max: resultMatrixHalfBW) min: matrixAHeight-1) do: [ :iLessOne |

				resultMatrixElements at: (jLessOne * resultMatrixBWLessOne) + iLessOne put:
					(resultMatrixElements at: (jLessOne * resultMatrixBWLessOne) + iLessOne) +
						((matrixAElements at: (iLessOne * matrixABWLessOne) + ijLessOne) *
							(matrixBElements at: (jLessOne * matrixBBWLessOne) + ijLessOne))
			]
		]
	]! !

!NumericLinealAlgebraPlugin methodsFor: 'primitives' stamp: 'jmv 7/15/2002 13:25'!
primFloatBandMatrixOrthogonalizeModifGramSchmidtStartingAtCol: jc mxH: height mxW: width mxElem: elements mxBW: bandWidth

	"Make self an orhonormal matrix, by making columns be linear combinations of the original ones.
	Substract to each column its projection over all the other ones, except for column jc that remains unchanged.

	Use a sligthly modified of the Modified Gram-Schmidt method. Instead of starting at column 1, can start on any column.
	(this can be useful to adjust witch columns should be less affected).
	The effect is the same as taking the columns 1..(jc-1), and moving them to the right of the matrix, placing first the jc-1
	column, and last the 1 column; performing the regular Modified Gram-Schmidt method; and placing again each column
	where it belongs. If jc = 1, it's just the regular method.

	Besides, each column is normalized.

	Based on the code in AbstractMatrix, but hand inlined and hand optimized"

	"Warning: if the band is not wide enough to hold the new matrix,
		ALL THE ELEMENTS OUSIDE THE BAND ARE SIMPLY DISCARDED, and the resulting matrix is not orthogonal!!"

	| halfBW kLessOneTimesBandWidthLessOne colKTop colKBottom normSquared norm colKIdx
		jLessOneTimesBandWidthLessOne colJTop colJBottom intersectionColsKJTop intersectionColsKJBottom
		correlationFactor colJIdx elem |

	self export: true.
	self
		primitive: 'primFloatBandMatrixOrthogonalizeModifGramSchmidtStartingAtCol'
		parameters: #(SmallInteger SmallInteger SmallInteger FloatArray SmallInteger)
		receiver:	#Oop.
	self var: #elem declareC: 'double elem'.
	self var: #normSquared declareC: 'double normSquared'.
	self var: #norm declareC: 'double norm'.
	self var: #correlationFactor declareC: 'double correlationFactor'.

	halfBW _ bandWidth - 1 / 2.
	jc to: width do: [ :k |
		"Normalize column k"
		kLessOneTimesBandWidthLessOne _ (k-1) * (bandWidth-1) -1.
		colKTop _ k - halfBW max: 1.
		colKBottom _ k + halfBW min: height.
		normSquared _ 0.0.
		colKTop to: colKBottom do: [ :i |
			elem _ (elements at: kLessOneTimesBandWidthLessOne + i).
			normSquared _ normSquared + (elem * elem) ].
		norm _ normSquared sqrt.
		norm > 0 ifTrue: [
			colKTop to: colKBottom do: [ :i |
				colKIdx _ kLessOneTimesBandWidthLessOne + i.
				elements at: colKIdx put: (elements at: colKIdx) / norm ] ].

		"Substract column k to all those at it's right"
		k + 1 to: width do: [ :j |
			"Substract to column j its projection over column k"
			jLessOneTimesBandWidthLessOne _ (j-1) * (bandWidth-1) -1.
			colJTop _ j - halfBW max: 1.
			colJBottom _ j + halfBW min: height.
			intersectionColsKJTop _ colKTop max: colJTop.
			intersectionColsKJBottom _ colKBottom min: colJBottom.
			"Find Correlation Factor"
			correlationFactor _ 0.
			intersectionColsKJTop to: intersectionColsKJBottom do: [ :i |
				correlationFactor _ correlationFactor + 
					((elements at: kLessOneTimesBandWidthLessOne + i) * (elements at: jLessOneTimesBandWidthLessOne + i)) ].
			"This generates bad C code (using abs() instead of fabs())"
			"correlationFactor abs > 0.0 ifTrue: ["
			correlationFactor * correlationFactor > 0.0 ifTrue: [
				"The intersection between the height of both columns, substract"
				intersectionColsKJTop to: intersectionColsKJBottom do: [ :i |
					colJIdx _ jLessOneTimesBandWidthLessOne + i.
					elements at: colJIdx put:
						(elements at: colJIdx) - ((elements at: kLessOneTimesBandWidthLessOne + i) * correlationFactor) ].
			].
		].

		jc-1 to: 1 by: -1 do: [ :j |
			"Substract to column j its projection over column k"
			jLessOneTimesBandWidthLessOne _ (j-1) * (bandWidth-1) -1.
			colJTop _ j - halfBW max: 1.
			colJBottom _ j + halfBW min: height.
			intersectionColsKJTop _ colKTop max: colJTop.
			intersectionColsKJBottom _ colKBottom min: colJBottom.
			"Find Correlation Factor"
			correlationFactor _ 0.
			intersectionColsKJTop to: intersectionColsKJBottom do: [ :i |
				correlationFactor _ correlationFactor + 
					((elements at: kLessOneTimesBandWidthLessOne + i) * (elements at: jLessOneTimesBandWidthLessOne + i)) ].
			"This generates bad C code (using abs() instead of fabs())"
			"correlationFactor abs > 0.0 ifTrue: ["
			correlationFactor * correlationFactor > 0.0 ifTrue: [
				"The intersection between the height of both columns, substract"
				intersectionColsKJTop to: intersectionColsKJBottom do: [ :i |
					colJIdx _ jLessOneTimesBandWidthLessOne + i.
					elements at: colJIdx put: 
						(elements at: colJIdx) - ((elements at: kLessOneTimesBandWidthLessOne + i) * correlationFactor) ].
			].
		].
	].

	jc-1 to: 1 by: -1 do: [ :k |
		"Normalize column k"
		kLessOneTimesBandWidthLessOne _ (k-1) * (bandWidth-1) -1.
		colKTop _ k - halfBW max: 1.
		colKBottom _ k + halfBW min: height.
		normSquared _ 0.0.
		colKTop to: colKBottom do: [ :i |
			elem _ (elements at: kLessOneTimesBandWidthLessOne + i).
			normSquared _ normSquared + (elem * elem) ].
		norm _ normSquared sqrt.
		norm > 0 ifTrue: [
			colKTop to: colKBottom do: [ :i |
				colKIdx _ kLessOneTimesBandWidthLessOne + i.
				elements at: colKIdx put: (elements at: colKIdx) / norm ] ].

		k - 1 to: 1 by: -1 do: [ :j |
			"Substract to column j its projection over column k"
			jLessOneTimesBandWidthLessOne _ (j-1) * (bandWidth-1) -1.
			colJTop _ j - halfBW max: 1.
			colJBottom _ j + halfBW min: height.
			intersectionColsKJTop _ colKTop max: colJTop.
			intersectionColsKJBottom _ colKBottom min: colJBottom.
			"Find Correlation Factor"
			correlationFactor _ 0.
			intersectionColsKJTop to: intersectionColsKJBottom do: [ :i |
				correlationFactor _ correlationFactor + 
					((elements at: kLessOneTimesBandWidthLessOne + i) * (elements at: jLessOneTimesBandWidthLessOne + i)) ].
			"This generates bad C code (using abs() instead of fabs())"
			"correlationFactor abs > 0.0 ifTrue: ["
			correlationFactor * correlationFactor > 0.0 ifTrue: [
				"The intersection between the height of both columns, substract"
				intersectionColsKJTop to: intersectionColsKJBottom do: [ :i |
					colJIdx _ jLessOneTimesBandWidthLessOne + i.
					elements at: colJIdx put: 
						(elements at: colJIdx) - ((elements at: kLessOneTimesBandWidthLessOne + i) * correlationFactor) ].
			].
		].
	].
! !

!NumericLinealAlgebraPlugin methodsFor: 'primitives' stamp: 'jmv 7/15/2002 13:22'!
primFloatMatrixOrthogonalizeModifGramSchmidtStartingAtCol: jc mxH: height mxW: width mxElem: elements

	"Make self an orhonormal matrix, by making columns be linear combinations of the original ones.
	Substract to each column its projection over all the other ones, except for column jc that remains unchanged.

	Use a sligthly modified of the Modified Gram-Schmidt method. Instead of starting at column 1, can start on any column.
	(this can be useful to adjust witch columns should be less affected).
	The effect is the same as taking the columns 1..(jc-1), and moving them to the right of the matrix, placing first the jc-1
	column, and last the 1 column; performing the regular Modified Gram-Schmidt method; and placing again each column
	where it belongs. If jc = 1, it's just the regular method.

	Besides, each column is normalized.

	Based on the code in AbstractMatrix, but hand inlined and hand optimized"

	| kLessOneTimesHeightLessOne normSquared norm colKIdx jLessOneTimesHeightLessOne correlationFactor colJIdx elem |

	self export: true.
	self
		primitive: 'primFloatMatrixOrthogonalizeModifGramSchmidtStartingAtCol'
		parameters: #(SmallInteger SmallInteger SmallInteger FloatArray)
		receiver:	#Oop.
	self var: #elem declareC: 'double elem'.
	self var: #normSquared declareC: 'double normSquared'.
	self var: #norm declareC: 'double norm'.
	self var: #correlationFactor declareC: 'double correlationFactor'.

	jc to: width do: [ :k |
		"Normalize column k"
		kLessOneTimesHeightLessOne _ (k-1) * height -1.
		normSquared _ 0.0.
		1 to: height do: [ :i |
			elem _ (elements at: kLessOneTimesHeightLessOne + i).
			normSquared _ normSquared + (elem * elem) ].
		norm _ normSquared sqrt.
		norm > 0 ifTrue: [
			1 to: height do: [ :i |
				colKIdx _ kLessOneTimesHeightLessOne + i.
				elements at: colKIdx put: (elements at: colKIdx) / norm ] ].

		"Substract column k to all those at it's right"
		k + 1 to: width do: [ :j |
			"Substract to column j its projection over column k"
			jLessOneTimesHeightLessOne _ (j-1) * height -1.
			"Find Correlation Factor"
			correlationFactor _ 0.
			1 to: height do: [ :i |
				correlationFactor _ correlationFactor + 
					((elements at: kLessOneTimesHeightLessOne + i) * (elements at: jLessOneTimesHeightLessOne + i)) ].
			"This generates bad C code (using abs() instead of fabs())"
			"correlationFactor abs > 0.0 ifTrue: ["
			correlationFactor * correlationFactor > 0.0 ifTrue: [
				1 to: height do: [ :i |
					colJIdx _ jLessOneTimesHeightLessOne + i.
					elements at: colJIdx put:
						(elements at: colJIdx) - ((elements at: kLessOneTimesHeightLessOne + i) * correlationFactor) ].
			].
		].

		jc-1 to: 1 by: -1 do: [ :j |
			"Substract to column j its projection over column k"
			jLessOneTimesHeightLessOne _ (j-1) * height -1.
			"Find Correlation Factor"
			correlationFactor _ 0.
			1 to: height do: [ :i |
				correlationFactor _ correlationFactor + 
					((elements at: kLessOneTimesHeightLessOne + i) * (elements at: jLessOneTimesHeightLessOne + i)) ].
			"This generates bad C code (using abs() instead of fabs())"
			"correlationFactor abs > 0.0 ifTrue: ["
			correlationFactor * correlationFactor > 0.0 ifTrue: [
				1 to: height do: [ :i |
					colJIdx _ jLessOneTimesHeightLessOne + i.
					elements at: colJIdx put: 
						(elements at: colJIdx) - ((elements at: kLessOneTimesHeightLessOne + i) * correlationFactor) ].
			].
		].
	].

	jc-1 to: 1 by: -1 do: [ :k |
		"Normalize column k"
		kLessOneTimesHeightLessOne _ (k-1) * height -1.
		normSquared _ 0.0.
		1 to: height do: [ :i |
			elem _ (elements at: kLessOneTimesHeightLessOne + i).
			normSquared _ normSquared + (elem * elem) ].
		norm _ normSquared sqrt.
		norm > 0 ifTrue: [
			1 to: height do: [ :i |
				colKIdx _ kLessOneTimesHeightLessOne + i.
				elements at: colKIdx put: (elements at: colKIdx) / norm ] ].

		k - 1 to: 1 by: -1 do: [ :j |
			"Substract to column j its projection over column k"
			jLessOneTimesHeightLessOne _ (j-1) * height -1.
			"Find Correlation Factor"
			correlationFactor _ 0.
			1 to: height do: [ :i |
				correlationFactor _ correlationFactor + 
					((elements at: kLessOneTimesHeightLessOne + i) * (elements at: jLessOneTimesHeightLessOne + i)) ].
			"This generates bad C code (using abs() instead of fabs())"
			"correlationFactor abs > 0.0 ifTrue: ["
			correlationFactor * correlationFactor > 0.0 ifTrue: [
				1 to: height do: [ :i |
					colJIdx _ jLessOneTimesHeightLessOne + i.
					elements at: colJIdx put: 
						(elements at: colJIdx) - ((elements at: kLessOneTimesHeightLessOne + i) * correlationFactor) ].
			].
		].
	].
! !

!NumericLinealAlgebraPlugin methodsFor: 'primitives' stamp: 'jmv 7/8/2002 14:47'!
primFloatVectorAddTo: resultMatrixElements size: resultMatrixSize resultOfBandMatrixMultiply: matrixAElements bw: matrixABW byVector: matrixBElements

	"Add to resultMatrix the result of matrixA * matrixB
		resultMatrix += matrixA * matrixB"
	"resultMatrix and matrixB are vectors, i.e. matrices of n*1. matrixA is a Float Square Band Matrix"
	"Hand inlined and hand optimized. The order of the iterations was choosed trying to minimize
		page faults in a virtual memory system when using large matrices."

	"resultMatrix should contain zeros before calling.
	For safety, matrix sizes should be got from the objects themselves, and the FloatArrays should
	be checked to be big enough"

	| matrixAHalfBW matrixABWLessOne |

	self export: true.
	self
		primitive: 'primFloatVectorAddTosizeresultOfBandMatrixMultiplybwbyVector'
		parameters: #(FloatArray SmallInteger FloatArray SmallInteger FloatArray)
		receiver:	#Oop.

	matrixABWLessOne _ matrixABW - 1.
	matrixAHalfBW _ matrixABW - 1 / 2.

	"Iterate ij over matrixA columns and matrixB rows"
	1-1 to: resultMatrixSize-1 do: [ :ijLessOne |

		"iterate the i's (rows) of resultMatrix, i is also a row of matrixA."
		(ijLessOne - matrixAHalfBW max: 1-1) to:
			(ijLessOne + matrixAHalfBW min: resultMatrixSize-1) do: [ :iLessOne |

			resultMatrixElements at: iLessOne put:
				(resultMatrixElements at: iLessOne) +
					((matrixAElements at: (ijLessOne * matrixABWLessOne) + iLessOne) *
						(matrixBElements at: ijLessOne))
		]
	]! !

!NumericLinealAlgebraPlugin methodsFor: 'primitives' stamp: 'jmv 7/8/2002 14:44'!
primFloatVectorAddTo: resultMatrixElements size: resultMatrixSize resultOfBandMatrixTransposedMultiply: matrixAElements bw: matrixABW byVector: matrixBElements

	"Add to resultMatrix the result of matrixA transposed * matrixB
		resultMatrix += matrixA transposed * matrixB"
	"resultMatrix and matrixB are vectors, i.e. matrices of n*1. matrixA is a Float Square Band Matrix"
	"Hand inlined and hand optimized. The order of the iterations was choosed trying to minimize
		page faults in a virtual memory system when using large matrices."

	"resultMatrix should contain zeros before calling.
	For safety, matrix sizes should be got from the objects themselves, and the FloatArrays should
	be checked to be big enough"

	| matrixAHalfBW matrixABWLessOne |

	self export: true.
	self
		primitive: 'primFloatVectorAddTosizeresultOfBandMatrixTransposedMultiplybwbyVector'
		parameters: #(FloatArray SmallInteger FloatArray SmallInteger FloatArray)
		receiver:	#Oop.

	matrixABWLessOne _ matrixABW - 1.
	matrixAHalfBW _ matrixABW - 1 / 2.

	"iterate the i's (rows) of resultMatrix, i is also a row of matrixA, and thus it's a column
		of matrixA transposed."
	1-1 to: resultMatrixSize-1 do: [ :iLessOne |

		"Iterate ij over matrixA columns and matrixB rows"
		"Matrix A height and width are equal, it is square (and band)"
		(iLessOne - matrixAHalfBW max: 1-1) to:
			(iLessOne + matrixAHalfBW min: resultMatrixSize-1) do: [ :ijLessOne |

			resultMatrixElements at: iLessOne put:
				(resultMatrixElements at: iLessOne) +
					((matrixAElements at: (iLessOne * matrixABWLessOne) + ijLessOne) *
						(matrixBElements at: ijLessOne))
		]
	]! !

!NumericLinealAlgebraPlugin methodsFor: 'primitives' stamp: 'jmv 7/12/2002 17:04'!
primFloatVectorAddTo: resultMatrixElements size: resultMatrixSize resultOfMatrixMultiply: matrixAElements byVector: matrixBElements size: matrixBSize

	"Add to resultMatrix the result of matrixA * matrixB
		resultMatrix += matrixA * matrixB"
	"resultMatrix and matrixB are vectors, i.e. matrices of n*1. matrixA is a Float Matrix"
	"Hand inlined and hand optimized. The order of the iterations was choosed trying to minimize
		page faults in a virtual memory system when using large matrices."

	"resultMatrix should contain zeros before calling.
	For safety, matrix sizes should be got from the objects themselves, and the FloatArrays should
	be checked to be big enough"

	self export: true.
	self
		primitive: 'primFloatVectorAddTosizeresultOfMatrixMultiplybyVectorsize'
		parameters: #(FloatArray SmallInteger FloatArray FloatArray SmallInteger)
		receiver:	#Oop.

	"Iterate ij over matrixA columns and matrixB rows"
	1-1 to: matrixBSize-1 do: [ :ijLessOne |

		"iterate the i's (rows) of resultMatrix, i is also a row of matrixA."
		1-1 to: resultMatrixSize-1 do: [ :iLessOne |

			resultMatrixElements at: iLessOne put:
				(resultMatrixElements at: iLessOne) +
					((matrixAElements at: (ijLessOne * resultMatrixSize) + iLessOne) *
						(matrixBElements at: ijLessOne))
		]
	]! !

!NumericLinealAlgebraPlugin methodsFor: 'primitives' stamp: 'jmv 7/12/2002 17:05'!
primFloatVectorAddTo: resultMatrixElements size: resultMatrixSize resultOfMatrixTransposedMultiply: matrixAElements byVector: matrixBElements size: matrixBSize

	"Add to resultMatrix the result of matrixA transposed * matrixB
		resultMatrix += matrixA transposed * matrixB"
	"resultMatrix and matrixB are vectors, i.e. matrices of n*1. matrixA is a Float Matrix"
	"Hand inlined and hand optimized. The order of the iterations was choosed trying to minimize
		page faults in a virtual memory system when using large matrices."

	"resultMatrix should contain zeros before calling.
	For safety, matrix sizes should be got from the objects themselves, and the FloatArrays should
	be checked to be big enough"

	self export: true.
	self
		primitive: 'primFloatVectorAddTosizeresultOfMatrixTransposedMultiplybyVectorsize'
		parameters: #(FloatArray SmallInteger FloatArray FloatArray SmallInteger)
		receiver:	#Oop.

	"iterate the i's (rows) of resultMatrix, i is also a row of matrixA, and thus it's a column
		of matrixA transposed."
	1-1 to: resultMatrixSize-1 do: [ :iLessOne |

		"Iterate ij over matrixA columns and matrixB rows"
		"Matrix A height and width are equal, it is square (and band)"
		1-1 to: matrixBSize-1 do: [ :ijLessOne |

			resultMatrixElements at: iLessOne put:
				(resultMatrixElements at: iLessOne) +
					((matrixAElements at: (iLessOne * resultMatrixSize) + ijLessOne) *
						(matrixBElements at: ijLessOne))
		]
	]! !


AbstractMatrix subclass: #SparseMatrix
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Numeric Linear Algebra'!
!SparseMatrix commentStamp: '<historical>' prior: 0!
My instances are Matrices whose elements are mostly zero. So, only nonzero elements are stored.!

